@@ -1,3 +1,5 @@
+/*	$calcurse: apoint.h,v 1.4 2006/09/14 15:06:54 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -26,33 +28,39 @@
 #define CALCURSE_APOINT_H
 
 #include <ncurses.h>
+#include "notify.h"
 
 #define HRMIN_SIZE 6
 #define MESG_MAXSIZE 256
 
-struct apoint_s {
-	struct apoint_s *next;
+typedef struct apoint_llist_node {
+	struct apoint_llist_node *next;
 	long start;		/* seconds since 1 jan 1970 */
 	long dur;		/* duration of the appointment in seconds */
 	char *mesg;
-};
+} apoint_llist_node_t;
 
-extern struct apoint_s *apointlist;
-extern struct apoint_s *adayptr;
+typedef struct apoint_llist {
+	apoint_llist_node_t *root;
+	pthread_mutex_t mutex;
+} apoint_llist_t;
 
-struct apoint_s *apoint_new(char *, long, long);
-struct apoint_s *adayadd(char *mesg, long start, long dur);
-int aday_store(long date);
-void free_aday(void);
-unsigned apoint_inday(struct apoint_s *o, long start);
-void apoint_sec2str(struct apoint_s *o, long day, char *start, char *end);
-void apoint_write(struct apoint_s *o, FILE * f);
-struct apoint_s *apoint_scan(FILE * f, struct tm start, struct tm end);
+extern apoint_llist_t *alist_p;
+
+int apoint_llist_init(void);
+apoint_llist_node_t *apoint_new(char *, long, long);
+unsigned apoint_inday(apoint_llist_node_t *o, long start);
+void apoint_sec2str(apoint_llist_node_t *o, 
+	int type, long day, char *start, char *end);
+void apoint_write(apoint_llist_node_t *o, FILE * f);
+apoint_llist_node_t *apoint_scan(FILE * f, struct tm start, struct tm end);
 void apoint_delete_bynum(long start, unsigned num);
-void display_item_date(WINDOW *win, int color, struct apoint_s *i,
-		long date, int y, int x);
+void display_item_date(WINDOW *win, int color, apoint_llist_node_t *i,
+		int type, long date, int y, int x);
 int get_item_line(int item_nb, int nb_events_inday);
 void scroll_pad_down(int item_nb, int nb_events_inday, int win_length);
 void scroll_pad_up(int item_nb, int nb_events_inday);
+struct notify_app_s *apoint_check_next(struct notify_app_s *app, long start);
+apoint_llist_node_t *apoint_recur_s2apoint_s(recur_apoint_llist_node_t *p);
 
 #endif /* CALCURSE_APOINT_H */
