@@ -1,3 +1,5 @@
+/*	$calcurse: args.c,v 1.7 2006/09/16 09:08:21 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -38,8 +40,11 @@
 #include "vars.h"
 #include "event.h"
 #include "apoint.h"
+#include "recur.h"
+#include "day.h"
 #include "todo.h"
 #include "io.h"
+#include "notify.h"
 
 /* 
  * Parse the command-line arguments and call the appropriate
@@ -49,11 +54,13 @@ int parse_args(int argc, char **argv, int colr)
 {
 	int ch, add_line = 0;
 	int unknown_flag = 0, app_found = 0;
-	int aflag = 0, cflag = 0, dflag = 0, vflag = 0, hflag = 0, tflag = 0;
+	int aflag = 0, cflag = 0, dflag = 0, vflag = 0; 
+	int hflag = 0, tflag = 0, nflag = 0;
 	int non_interactive = 0, multiple_flag = 0, load_data = 0;
+	int no_file = 1;
 	char *ddate = "", *cfile = NULL;
 
-	while ((ch = getopt(argc, argv, "hvtad:c:")) != -1) {
+	while ((ch = getopt(argc, argv, "hvntad:c:")) != -1) {
 		switch (ch) {
 		case 'a':
 			aflag = 1;
@@ -75,6 +82,11 @@ int parse_args(int argc, char **argv, int colr)
 		case 'h':
 			hflag = 1;
 			break;
+		case 'n':
+			nflag = 1;
+			multiple_flag++;
+			load_data++;
+			break;
 		case 't':
 			tflag = 1;
 			multiple_flag++;
@@ -109,12 +121,18 @@ int parse_args(int argc, char **argv, int colr)
 		} else if (multiple_flag) {
 			if (load_data) {
 				io_init(cfile);
-				if (dflag || aflag) load_app(colr);
+				no_file = check_data_files();
+				if (dflag || aflag || nflag)  
+					load_app(colr);
 			}
 			if (tflag) {
 				todo_arg(colr);
 				non_interactive = 1;
 			}
+			if (nflag) {
+				next_arg();
+				non_interactive = 1;
+			}
 			if (dflag) {
 				date_arg(ddate, add_line);
 				non_interactive = 1;
@@ -125,6 +143,7 @@ int parse_args(int argc, char **argv, int colr)
 		} else {
 			non_interactive = 0;
 			io_init(cfile);
+			no_file = check_data_files();
 		}
 		return non_interactive;
 	}
@@ -160,7 +179,10 @@ void help_arg()
 		"\nNon-interactive:\n"
 		"  -a 		print events and appointments for current day and exit.\n"
 		"  -d <date|num>	print events and appointments for <date> "
-		"or <num> upcoming\n\t\tdays and exit.\n"
+		"or <num> upcoming\n\t\tdays and exit. Possible formats are: "
+		"'mm/dd/yyyy' or 'n'.\n"
+		"  -n  		print next appointment within upcoming 24 hours "
+		"and exit.\n"
 		"  -t		print todo list and exit.\n"
 	    	"\nFor more information, type '?' from within Calcurse, "
 		"or read the manpage.\n"
@@ -179,16 +201,44 @@ void todo_arg(int colr)
 {
 	struct todo_s *i;
 	int nb_tod;
+	char priority[MAX_LENGTH] = "";
 
 	nb_tod = load_todo(colr);
 	fputs(_("to do:\n"),stdout);
 	for (i = todolist; i != 0; i = i->next) {
-		fputs(" - ",stdout);
+		sprintf(priority, "%d. ", i->id);	
+		fputs(priority,stdout);
 		fputs(i->mesg,stdout);
 		fputs("\n",stdout);
 	}
 }
 
+/* Print the next appointment within the upcoming 24 hours. */
+void next_arg(void)
+{
+	struct notify_app_s *next_app;
+	long current_time;
+	int time_left, hours_left, min_left;
+	char mesg[MAX_LENGTH];
+
+	current_time = now();
+	next_app = (struct notify_app_s *) malloc(sizeof(struct notify_app_s));
+	next_app->time = current_time + DAYINSEC;
+	next_app->got_app = 0;
+	next_app = recur_apoint_check_next(next_app, current_time, today());
+	next_app = apoint_check_next(next_app, current_time);
+	time_left = next_app->time - current_time;
+	if (time_left > 0 && time_left < DAYINSEC) {
+		hours_left = (time_left / 3600);
+		min_left = (time_left - hours_left*3600) / 60;
+		fputs(_("next appointment:\n"), stdout);
+		sprintf(mesg, "   [%02d:%02d] %s\n", hours_left, min_left, 
+			next_app->txt);
+		fputs(mesg, stdout);
+		fputs("\n", stdout);
+	}
+}
+
 /*
  * Print appointments for given day and exit.
  * If no year, month, and day is given, the given date is used.
@@ -196,8 +246,10 @@ void todo_arg(int colr)
  */
 int app_arg(int add_line, int year, int month, int day, long date)
 {
+	struct recur_event_s *re;
 	struct event_s *j;
-	struct apoint_s *i;
+	recur_apoint_llist_node_t *ra;
+	apoint_llist_node_t *i;
 	long today;
 	bool print_date = true;
 	int  app_found = 0;
@@ -213,6 +265,23 @@ int app_arg(int add_line, int year, int month, int day, long date)
  	 * that date and it is the first one, and then print all the events for
  	 * that date. 
 	 */
+	for (re = recur_elist; re != 0; re = re->next) {
+		if (recur_item_inday(re->day, re->exc, re->rpt->type, 
+			re->rpt->freq, re->rpt->until, today)) {
+			app_found = 1;
+			if (add_line) {
+				fputs("\n", stdout);
+				add_line = 0;
+			}
+			if (print_date) {
+				arg_print_date(today);
+				print_date = false;
+			}
+			fputs(" o ", stdout);
+			fputs(re->mesg, stdout); fputs("\n", stdout);
+		}
+	}
+
 	for (j = eventlist; j != 0; j = j->next) {
 		if (event_inday(j, today)) {
 			app_found = 1;
@@ -230,7 +299,33 @@ int app_arg(int add_line, int year, int month, int day, long date)
 	}
 
  	 /* Same process is performed but this time on the appointments. */
-	for (i = apointlist; i != 0; i = i->next) {
+	pthread_mutex_lock(&(recur_alist_p->mutex));
+	for (ra = recur_alist_p->root; ra != 0; ra = ra->next) {
+		if (recur_item_inday(ra->start, ra->exc, ra->rpt->type, 
+			ra->rpt->freq, ra->rpt->until, today)) {
+			app_found = 1;
+			if (add_line) {
+				fputs("\n",stdout);
+				add_line = 0;
+			}
+			if (print_date) {
+				arg_print_date(today);
+				print_date = false;
+			}
+			apoint_sec2str(apoint_recur_s2apoint_s(ra),
+				RECUR_APPT, today, apoint_start_time,
+				apoint_end_time);
+			fputs(" - ",stdout);
+			fputs(apoint_start_time,stdout);
+			fputs(" -> ",stdout);
+			fputs(apoint_end_time,stdout); fputs("\n\t",stdout);
+			fputs(ra->mesg,stdout); fputs("\n",stdout);
+		}
+	}
+	pthread_mutex_unlock(&(recur_alist_p->mutex));
+
+	pthread_mutex_lock(&(alist_p->mutex));
+	for (i = alist_p->root; i != 0; i = i->next) {
 		if (apoint_inday(i, today)) {
 			app_found = 1;
 			if (add_line) {
@@ -241,7 +336,7 @@ int app_arg(int add_line, int year, int month, int day, long date)
 				arg_print_date(today);
 				print_date = false;
 			}
-			apoint_sec2str(i, today, apoint_start_time,
+			apoint_sec2str(i, APPT, today, apoint_start_time,
 					apoint_end_time);
 			fputs(" - ",stdout);
 			fputs(apoint_start_time,stdout);
@@ -250,6 +345,8 @@ int app_arg(int add_line, int year, int month, int day, long date)
 			fputs(i->mesg,stdout); fputs("\n",stdout);
 		}	
 	}
+	pthread_mutex_unlock(&(alist_p->mutex));
+
 	return app_found;
 }
 
@@ -272,7 +369,7 @@ void date_arg(char *ddate, int add_line)
 	 * was entered, and then call app_arg() to print appointments
 	 */
 	arg_len = strlen(ddate);
-	if (arg_len <= 4) {
+	if (arg_len <= 4) { 		/* a number of days was entered */
 		for (i = 0; i <= arg_len-1; i++) { 
 			if (isdigit(ddate[i])) num_digit++;	
 		}
@@ -290,7 +387,7 @@ void date_arg(char *ddate, int add_line)
 			add_line = app_found;
 			ind = ind + sec_in_day;
 		}
-	} else {
+	} else {			/* a date was entered */
 		date_valid = check_date(ddate);
 		if (date_valid) {
 			sscanf(ddate, "%d / %d / %d", &month, &day, &year);
@@ -315,10 +412,9 @@ int
 check_date(char *date)
 {
 	int ok = 0;
-	char *month = "  ";
-	char *day = "  ";
-	char *year = "    ";
-
+	char month[3] = "";
+	char day[3] = "";
+	char year[5] = "";
 	if ( 
 			(strlen(date) == 10) &
 			(isdigit(date[0]) != 0) &
@@ -367,7 +463,7 @@ void arg_print_date(long date)
 void usage()
 {
         char *arg_usage = 
-                _("Usage: calcurse [-h | -v] [-at] [-d date|num] [-c file]\n");
+                _("Usage: calcurse [-h | -v] [-ant] [-d date|num] [-c file]\n");
 	
         fputs(arg_usage, stdout);
 }
