@@ -1,3 +1,5 @@
+/*	$calcurse: args.h,v 1.2 2006/09/16 09:09:06 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -31,6 +33,7 @@ int parse_args(int argc, char **argv, int colr);
 void version_arg();
 void help_arg();
 void todo_arg(int colr);
+void next_arg(void);
 int app_arg(int add_line, int year, int month, int day, long date);
 void date_arg(char *ddate, int add_line);
 int check_date(char *date);
