@@ -1,3 +1,5 @@
+/*	$calcurse: calcurse.c,v 1.23 2006/09/30 14:23:07 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -27,6 +29,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <ncurses.h>	
+#include <pthread.h>
 #include <time.h>
 #include <string.h>
 #include <stdlib.h>
@@ -34,6 +37,7 @@
 #include <math.h>
 #include <locale.h>
 
+#include "apoint.h"
 #include "i18n.h"
 #include "io.h"
 #include "help.h"
@@ -41,10 +45,12 @@
 #include "custom.h"
 #include "utils.h"
 #include "vars.h"
-#include "apoint.h"
+#include "day.h"
 #include "event.h"
+#include "recur.h"
 #include "todo.h"
 #include "args.h"
+#include "notify.h"
 
 
 /* Variables for calendar */
@@ -63,9 +69,6 @@ int number_apoints_inday;
 int number_events_inday;
 int first_app_onscreen = 0;
 int hilt_app = 0, sav_hilt_app;
-bool event_hilt = true;
-char saved_a_start[100], saved_a_end[100];
-char *saved_a_mesg, *saved_e_mesg;
 
 /* Variables for todo list */
 int nb_tod = 0, hilt_tod = 0, sav_hilt_tod;
@@ -86,29 +89,29 @@ bool week_begins_on_monday;
 /* 
  * Variables to handle calcurse windows 
  */ 
-int x_cal, y_cal, x_app, y_app, x_tod, y_tod, x_bar, y_bar;
-int nl_cal, nc_cal, nl_app, nc_app, nl_tod, nc_tod, nl_bar, nc_bar;
+int x_cal, y_cal, x_app, y_app, x_tod, y_tod, x_bar, y_bar, x_not, y_not;
+int nl_cal, nc_cal, nl_app, nc_app, nl_tod, nc_tod;
+int nl_bar, nc_bar, nl_not, nc_not;
 int which_pan = 0;
 enum window_number {CALENDAR, APPOINTMENT, TODO};
 
 /* External functions */
 void get_date(void);
 void init_vars(int colr);
+void init_notify_bar(void);
 void init_wins(void), reinit_wins(void);
 void add_item(void);
-void add_todo(void);
 void load_conf(void);
 bool fill_config_var(char *string);
 void update_todo_panel(void);
 void update_app_panel(int yeat, int month, int day);
 void store_day(int year, int month, int day, bool day_changed);
-void write_app_pad(long date, int width, int length, int incolor, int colr);
 void get_screen_config(void);
 void update_windows(int surrounded_window);
-void general_config(void);
-void print_general_options(WINDOW *win);
+void general_config(void), config_notify_bar(void);
+void print_general_options(WINDOW *win), print_notify_options(WINDOW *win);
 void print_option_incolor(WINDOW *win, bool option, int pos_x, int pos_y);
-void del_apoint(void);
+void del_item(void);
 
 /*
  * Calcurse  is  a text-based personal organizer which helps keeping track
@@ -135,6 +138,10 @@ int main(int argc, char **argv)
         textdomain (PACKAGE);
 #endif /* ENABLE_NLS */
 		
+	/* Thread-safe data structure init */
+	apoint_llist_init();
+	recur_apoint_llist_init();
+
 	/* 
 	 * Begin by parsing and handling command line arguments.
 	 * The data path is also initialized here.
@@ -148,6 +155,7 @@ int main(int argc, char **argv)
 	noecho();		/* controls echoing of typed chars */
 	curs_set(0);		/* make cursor invisible */
         get_date();
+	init_notify_bar();
 	get_screen_config();
 	
         /* Check if terminal supports color. */
@@ -172,6 +180,8 @@ int main(int argc, char **argv)
 
 	init_vars(colr);
 	init_wins();
+	notify_init_bar(nl_not, nc_not, y_not, x_not);
+	reset_status_page();
 	update_windows(which_pan);
 
 	/* 
@@ -184,6 +194,7 @@ int main(int argc, char **argv)
 	custom_init_attr(colr);
 	nb_tod = load_todo(colr);	
 	load_app();
+	if (notify_bar()) notify_start_main_thread();
 	get_screen_config();
         reinit_wins();
         startup_screen(skip_system_dialogs, no_data_file, colr);
@@ -208,6 +219,7 @@ int main(int argc, char **argv)
 		switch (ch) {
 
 		case 9:	/* The TAB key was hit. */
+			reset_status_page();
 			/* Save previously highlighted event. */
 			if (which_pan == TODO) {
 				sav_hilt_tod = hilt_tod;
@@ -237,11 +249,15 @@ int main(int argc, char **argv)
 			}
 			break;
 
-		case 'R':
-		case 'r':	
+		case CTRL('R'):
                         reinit_wins();
 			break;
 
+		case 'O':
+		case 'o':
+			other_status_page(which_pan);
+			break;
+
 		case 'G':
 		case 'g':	/* Goto function */
 			erase_window_part(swin, 0, 0, nc_bar, nl_bar);
@@ -254,14 +270,8 @@ int main(int argc, char **argv)
 
 		case 'V':
 		case 'v':	/* View function */
-			if ((which_pan == APPOINTMENT) & (hilt_app != 0)) {
-				if (event_hilt) 
-					item_in_popup(NULL, NULL, 
-						saved_e_mesg, _("Event :"));
-				else
-					item_in_popup(saved_a_start, saved_a_end,
-					saved_a_mesg, _("Appointment :"));
-			}
+			if ((which_pan == APPOINTMENT) & (hilt_app != 0))
+				day_popup_item();
 			else if ((which_pan == TODO) & (hilt_tod != 0)) 
 				item_in_popup(NULL, NULL, saved_t_mesg,
 						_("To do :"));
@@ -297,6 +307,10 @@ int main(int argc, char **argv)
 				case 'g':
 					general_config();
 					break;
+				case 'N':
+				case 'n':
+					config_notify_bar();
+					break;
 				}
                                 reinit_wins();
 				erase_window_part(swin, 0, 0, nc_bar, nl_bar);
@@ -305,21 +319,47 @@ int main(int argc, char **argv)
                         update_windows(which_pan);
 			break;
 
+		case CTRL('A'):
+		case CTRL('T'):
 		case 'A':
 		case 'a':	/* Add an item */
-			if (which_pan == APPOINTMENT) {
+			if (which_pan == APPOINTMENT || ch == CTRL('A')) {
 				add_item();
 				do_storage = true;
+			} else if (which_pan == TODO || ch == CTRL('T')) {
+				nb_tod = todo_new_item(nb_tod, colr);
+				if (hilt_tod == 0 && nb_tod == 1)
+					hilt_tod++;
 			}
-			if (which_pan == TODO) add_todo();
 			break;
 
 		case 'D':
 		case 'd':	/* Delete an item */
-			del_apoint();
+			del_item();
 			do_storage = true;
 			break;
 
+		case 'R':
+		case 'r':
+			if (which_pan == APPOINTMENT && hilt_app != 0)
+				recur_repeat_item(sel_year, sel_month, 
+					sel_day, hilt_app, colr);
+				do_storage = true;
+			break;
+	
+		case '+':
+		case '-':
+			if (which_pan == TODO && hilt_tod != 0) {
+				hilt_tod = todo_chg_priority(ch, hilt_tod);
+				if (hilt_tod < first_todo_onscreen)
+					first_todo_onscreen = hilt_tod;
+				else if (hilt_tod - first_todo_onscreen >=
+				    nl_tod - 4)
+					first_todo_onscreen = hilt_tod 
+					    - nl_tod + 5;	
+			}
+			break;
+
 		case '?':	/* Online help system */
 			status_bar(which_pan, colr, nc_bar, nl_bar);
 			help_screen(which_pan, colr);
@@ -336,7 +376,8 @@ int main(int argc, char **argv)
 		case (261):	/* right arrow */
 		case ('L'):
 		case ('l'):
-			if (which_pan == CALENDAR) {
+		case CTRL('L'):
+			if (which_pan == CALENDAR || ch == CTRL('L')) {
 				do_storage = true;
 				day_changed = true;
 				if ((sel_day == 31) & (sel_month == 12))
@@ -357,7 +398,8 @@ int main(int argc, char **argv)
 		case (260):	/* left arrow */
 		case ('H'):
 		case ('h'):
-			if (which_pan == CALENDAR) {
+		case CTRL('H'):
+			if (which_pan == CALENDAR || ch == CTRL('H')) {
 				do_storage = true;
 				day_changed = true;
 				if ((sel_day == 1) & (sel_month == 1))
@@ -378,7 +420,8 @@ int main(int argc, char **argv)
 		case (259):	/* up arrow */
 		case ('K'):
 		case ('k'):
-			if (which_pan == CALENDAR) {
+		case CTRL('K'):
+			if (which_pan == CALENDAR || ch == CTRL('K')) {
 				do_storage = true;
 				day_changed = true;
 				if ((sel_day <= 7) & (sel_month == 1))
@@ -412,7 +455,8 @@ int main(int argc, char **argv)
 		case (258):	/* down arrow */
 		case ('J'):
 		case ('j'):
-			if (which_pan == CALENDAR) {
+		case CTRL('J'):
+			if (which_pan == CALENDAR || ch == CTRL('J')) {
 				do_storage = true;
 				day_changed = true;
 				if ((sel_day > days[sel_month - 1] - 7) & 
@@ -442,7 +486,7 @@ int main(int argc, char **argv)
 					++hilt_tod;
 					if (hilt_tod - first_todo_onscreen ==
 					    nl_tod - 4)
-						++first_todo_onscreen;
+						first_todo_onscreen++;
 				}
 			}
 			break;
@@ -514,6 +558,22 @@ void init_vars(int colr)
 	custom_init_attr(colr);
 }
 
+/* Notify-bar init */
+void init_notify_bar(void)
+{
+	char *time_format = "%T";
+	char *date_format = "%a %F";
+
+	nbar = (struct nbar_s *) malloc(sizeof(struct nbar_s));
+	nbar->datefmt = (char *) malloc(sizeof(char));
+	nbar->timefmt = (char *) malloc(sizeof(char));
+	pthread_mutex_init(&nbar->mutex, NULL);
+	nbar->show = 1;
+	nbar->cntdwn = 300; 
+	strncpy(nbar->datefmt, date_format, strlen(date_format) + 1); 
+	strncpy(nbar->timefmt, time_format, strlen(time_format) + 1);
+}
+
 /* 
  * Update all of the three windows and put a border around the
  * selected window.
@@ -543,6 +603,7 @@ void update_windows(int surrounded_window)
 			 sel_year, sel_day, day, month, year,
                          week_begins_on_monday);
 	status_bar(surrounded_window, colr, nc_bar, nl_bar);
+	if (notify_bar()) notify_update_bar();
         wmove(swin, 0, 0);
 	doupdate();
 }
@@ -555,16 +616,29 @@ void get_screen_config(void)
 	/* Get the screen configuration */
 	getmaxyx(stdscr, row, col);
 
-	/* window size definition */
+	/* fixed values for status, notification bars and calendar */
+	nl_bar = 2; nc_bar = col;
+	y_bar = row - nl_bar; x_bar = 0;
+	if (notify_bar()) {
+		nl_not = 1; nc_not = col;
+		y_not = y_bar - 1; x_not = 0;
+	} else {
+		nl_not = nc_not = y_not = x_not = 0;
+	}
 	nl_cal = 12;
 	nc_cal = 30;
-	nc_app = col - nc_cal;
-	nl_app = row - 2;
-	nc_tod = nc_cal;
-	nl_tod = row - (nl_cal + 2);
-	nl_app = row - 2;
-	nl_bar = 2; y_bar = row - 2;
-	nc_bar = col; x_bar = 0;
+
+	if (layout <= 4) { /* APPOINTMENT is the biggest panel */
+		nc_app = col - nc_cal;
+		nl_app = row - (nl_bar + nl_not);
+		nc_tod = nc_cal;
+		nl_tod = row - (nl_cal + nl_bar + nl_not);
+	} else { /* TODO is the biggest panel */
+		nc_tod = col - nc_cal;
+		nl_tod = row - (nl_bar + nl_not);
+		nc_app = nc_cal;
+		nl_app = row - (nl_cal + nl_bar + nl_not);
+	}
 
 	/* defining the layout */
 	switch (layout) {
@@ -584,6 +658,22 @@ void get_screen_config(void)
 		y_app = 0; x_tod = 0; y_tod = 0; x_cal = 0;
 		x_app = nc_cal; y_cal = nl_tod;
 		break;
+	case 5:
+		y_tod = 0; x_tod = 0; y_cal = 0;
+		y_app = nl_cal; x_app = nc_tod; x_cal = nc_tod;
+		break;
+	case 6:
+		y_tod = 0; x_tod = 0; y_app = 0;
+		x_app = nc_tod; x_cal = nc_tod; y_cal = nl_app;
+		break;
+	case 7:
+		y_tod = 0; x_app = 0; x_cal = 0; y_cal = 0;
+		x_tod = nc_cal; y_app = nl_cal;
+		break;
+	case 8:
+		y_tod = 0; x_app = 0; x_cal = 0; y_app = 0;
+		x_tod = nc_cal; y_cal = nl_app;
+		break;
 	}
 }
 
@@ -644,6 +734,7 @@ void reinit_wins(void)
         delwin(twin);
         get_screen_config();
         init_wins();
+	if (notify_bar()) notify_reinit_bar(nl_not, nc_not, y_not, x_not);
         update_windows(which_pan);
 }
 
@@ -653,10 +744,11 @@ void general_config(void)
 	WINDOW *conf_win;
 	char label[80];
 	char *number_str = _("Enter an option number to change its value [Q to quit] ");
-	int ch;
+	int ch, win_row;
 
 	clear();
-	conf_win = newwin(row - 2, col, 0, 0);
+	win_row = (notify_bar()) ? row - 3 : row - 2;
+	conf_win = newwin(win_row, col, 0, 0);
 	box(conf_win, 0, 0);
 	sprintf(label, _("CalCurse %s | general options"), VERSION);
 	win_show(conf_win, label);
@@ -691,6 +783,85 @@ void general_config(void)
 	delwin(conf_win);
 }
 
+/* Configuration for the notify-bar */
+void config_notify_bar(void)
+{
+	WINDOW *conf_win;
+	char label[80];
+	char buf[MAX_LENGTH];
+	char *number_str = _("Enter an option number to change its value [Q to quit] ");
+	char *date_str = 
+		_("Enter the date format (see 'man 3 strftime' for possible formats) ");
+	char *time_str = 
+		_("Enter the time format (see 'man 3 strftime' for possible formats) ");
+	char *count_str = 
+		_("Enter the number of seconds (0 not to be warned before an appointment)");
+	int ch = 0 , win_row, change_win = 1;
+
+	win_row = (notify_bar()) ? row - 3 : row - 2;
+	sprintf(label, _("CalCurse %s | notify-bar options"), VERSION);
+	while (ch != 'q') {
+		if (change_win) {
+			clear();
+			conf_win = newwin(win_row, col, 0, 0);
+			box(conf_win, 0, 0);
+			win_show(conf_win, label);
+		}
+		status_mesg(number_str, "");
+		print_notify_options(conf_win);
+		ch = wgetch(swin);
+
+		switch (ch) {
+		case '1':	
+			pthread_mutex_lock(&nbar->mutex);
+			nbar->show = !nbar->show;
+			pthread_mutex_unlock(&nbar->mutex);
+			notify_stop_main_thread();
+			if (notify_bar()) {
+				notify_start_main_thread();
+				win_row = row - 3;
+			} else {
+				win_row = row - 2;
+			}
+			delwin(conf_win);
+			change_win = 1;
+			break;
+		case '2':
+			status_mesg(date_str, "");
+			getstring(swin, colr, buf, 0, 1);
+			if (strlen(buf) != 0) {
+				pthread_mutex_lock(&nbar->mutex);
+				strncpy(nbar->datefmt, buf, strlen(buf) + 1);
+				pthread_mutex_unlock(&nbar->mutex);
+			}
+			change_win = 0;
+			break;
+		case '3':
+			status_mesg(time_str, "");
+			getstring(swin, colr, buf, 0, 1);
+			if (strlen(buf) != 0 ) {
+				pthread_mutex_lock(&nbar->mutex);
+				strncpy(nbar->timefmt, buf, strlen(buf) + 1);
+				pthread_mutex_unlock(&nbar->mutex);
+			}
+			change_win = 0;
+			break;
+                case '4':
+			status_mesg(count_str, "");
+			getstring(swin, colr, buf, 0, 1);
+			if (strlen(buf) != 0 && is_all_digit(buf) &&
+			    atoi(buf) >= 0 && atoi(buf) <= DAYINSEC) {
+				pthread_mutex_lock(&nbar->mutex);
+				nbar->cntdwn = atoi(buf);
+				pthread_mutex_unlock(&nbar->mutex);
+			}
+			change_win = 0;
+                        break;
+		}
+	}
+	delwin(conf_win);
+}
+
 /* prints the general options */
 void print_general_options(WINDOW *win)
 {
@@ -703,7 +874,7 @@ void print_general_options(WINDOW *win)
         char *option6 = _("week_begins_on_monday = ");
 
 	x_pos = 3;
-	y_pos = 4;
+	y_pos = 3;
 
 	mvwprintw(win, y_pos, x_pos, "[1] %s      ", option1);
 	print_option_incolor(win, auto_save, y_pos,
@@ -746,6 +917,52 @@ void print_general_options(WINDOW *win)
 	doupdate();
 }
 
+/* prints options related to the notify-bar */
+void print_notify_options(WINDOW *win)
+{
+	int x_pos, y_pos;
+	char *option1 = _("notify-bar_show = ");
+	char *option2 = _("notify-bar_date = ");
+	char *option3 = _("notify-bar_clock = ");
+        char *option4 = _("notify-bar_warning = ");
+
+	x_pos = 3;
+	y_pos = 4;
+
+	pthread_mutex_lock(&nbar->mutex);
+	mvwprintw(win, y_pos, x_pos, "[1] %s      ", option1);
+	print_option_incolor(win, nbar->show, y_pos,
+			     x_pos + 4 + strlen(option1));
+	mvwprintw(win, y_pos + 1, x_pos,
+		 _("(if set to YES, notify-bar will be displayed)"));
+
+	mvwprintw(win, y_pos + 3, x_pos, "[2] %s                       ", option2);
+	custom_apply_attr(win, ATTR_HIGHEST);
+	mvwprintw(win, y_pos + 3, x_pos + 4 + strlen(option2), "%s", nbar->datefmt);
+	custom_remove_attr(win, ATTR_HIGHEST);
+	mvwprintw(win, y_pos + 4, x_pos,
+		 _("(Format of the date to be displayed inside notify-bar)"));
+
+	mvwprintw(win, y_pos + 6, x_pos, "[3] %s                       ", option3);
+	custom_apply_attr(win, ATTR_HIGHEST);
+	mvwprintw(win, y_pos + 6, x_pos + 4 + strlen(option3), "%s", nbar->timefmt);
+	custom_remove_attr(win, ATTR_HIGHEST);
+	mvwprintw(win, y_pos + 7, x_pos,
+		 _("(Format of the time to be displayed inside notify-bar)"));
+        
+	mvwprintw(win, y_pos + 9, x_pos, "[4] %s                       ", option4);
+	custom_apply_attr(win, ATTR_HIGHEST);
+	mvwprintw(win, y_pos + 9, x_pos + 4 + strlen(option4), "%d", nbar->cntdwn);
+	custom_remove_attr(win, ATTR_HIGHEST);
+	mvwprintw(win, y_pos + 10, x_pos,
+		 _("(Warn user if an appointment is within next 'notify-bar_warning' seconds)"));
+
+	pthread_mutex_unlock(&nbar->mutex);
+	wmove(swin, 1, 0);
+	wnoutrefresh(win);
+	doupdate();
+}
+
 /* print the option value with appropriate color */
 void print_option_incolor(WINDOW *win, bool option, int pos_y, int pos_x)
 {
@@ -775,7 +992,7 @@ void print_option_incolor(WINDOW *win, bool option, int pos_y, int pos_x)
 }
 
   /* Delete an event from the ToDo or Appointment lists */
-void del_apoint(void)
+void del_item(void)
 {
 	char *choices = "[y/n] ";
 	char *del_app_str = _("Do you really want to delete this item ?");
@@ -784,8 +1001,9 @@ void del_apoint(void)
 	int nb_items = number_apoints_inday + number_events_inday;
 	bool go_for_deletion = false;
 	bool go_for_todo_del = false;
-	int to_be_removed = 0;
+	int to_be_removed;
 	int answer = 0;
+	int deleted_item_type = 0;
 	
 	/* delete an appointment */
 	if (which_pan == APPOINTMENT && hilt_app != 0) {
@@ -806,17 +1024,22 @@ void del_apoint(void)
 		
 		if (go_for_deletion) {
 			if (nb_items != 0) {
-				if (hilt_app <= number_events_inday) { 
-					event_delete_bynum(date, hilt_app - 1);
+				deleted_item_type = day_erase_item(date, hilt_app);
+				if (deleted_item_type == EVNT || 
+				    deleted_item_type == RECUR_EVNT) {
 					number_events_inday--;
 					to_be_removed = 1;
-				} else {
-					apoint_delete_bynum(date, 
-							hilt_app - 
-							number_events_inday - 1);
+				} else if (deleted_item_type == APPT ||
+				    deleted_item_type == RECUR_APPT) {
 					number_apoints_inday--;
 					to_be_removed = 3;
-				}
+				} else if (deleted_item_type == 0) {
+					to_be_removed = 0;		
+				} else { /* NOTREACHED */
+					fputs(_("FATAL ERROR in del_item: no such type\n"), stderr);
+					exit(EXIT_FAILURE);
+				}	
+
 				if (hilt_app > 1) --hilt_app;
 				if (apad->first_onscreen >= to_be_removed)
 					apad->first_onscreen = 
@@ -848,24 +1071,6 @@ void del_apoint(void)
 	}
 }
 
-  /* Add an item in the ToDo list */
-void add_todo(void)
-{
-	char *mesg = _("Enter the new ToDo item : ");
-	char todo_input[500];
-
-	status_mesg(mesg, "");
-	getstring(swin, colr, todo_input, 0, 1);
-	if (strlen(todo_input) != 0) {
-		todo_insert(todo_input);
-		++nb_tod;
-		update_todo_panel();
-	}
-	erase_window_part(swin, 0, 0, nc_bar, nl_bar);
-	status_bar(which_pan, colr, nc_bar, nl_bar);
-	doupdate();
-}
-
 /* 
  * Add an item in either the appointment or the event list,
  * depending if the start time is entered or not.
@@ -879,18 +1084,18 @@ void add_item(void)
 	char *format_message_2 = _("You entered an invalid end time, should be [h:mm] or [hh:mm] or [mm]");
         char *enter_str = _("Press [Enter] to continue");
 	int Id;
-        char item_time[500];
-	char item_mesg[500];
-	long apoint_duration;
-	struct apoint_s *apoint_pointeur;
+        char item_time[MAX_LENGTH];
+	char item_mesg[MAX_LENGTH];
+	long apoint_duration, apoint_start;
+	apoint_llist_node_t *apoint_pointeur;
         struct event_s *event_pointeur;
 	unsigned heures, minutes;
 	unsigned end_h, end_m;
         int is_appointment = 1;
 
 	/* Get the starting time */
-	strcpy(item_time, "     ");
-	while (check_time(item_time) == 0) {
+	strncpy(item_time, "     ", 6);
+	while (check_time(item_time) != 1) {
                 status_mesg(mesg_1, "");
 		getstring(swin, colr, item_time, 0, 1);
 		if (strlen(item_time) == 0){
@@ -908,7 +1113,7 @@ void add_item(void)
          * corresponding item.
          */
         if (is_appointment){ /* Get the appointment duration */
-                strcpy(item_time, " ");
+                strncpy(item_time, "     ", 6);
                 while (check_time(item_time) == 0) {
                         status_mesg(mesg_2, "");
                         getstring(swin, colr, item_time, 0, 1);
@@ -944,11 +1149,11 @@ void add_item(void)
 	if (strlen(item_mesg) != 0) {
                 if (is_appointment){
 		// insert the appointment in list
+		apoint_start = date2sec(sel_year, sel_month, sel_day,
+			heures, minutes);
 		apoint_pointeur =
-		    apoint_new(item_mesg,
-			      date2sec(sel_year, sel_month, sel_day,
-				       heures, minutes),
-			      min2sec(apoint_duration));
+		    apoint_new(item_mesg, apoint_start, min2sec(apoint_duration));
+		if (notify_bar()) notify_check_added(item_mesg, apoint_start);
                 // insert the event in list
                 } else {
                         event_pointeur = event_new(item_mesg, date2sec(
@@ -958,11 +1163,9 @@ void add_item(void)
                                                            12, 0),
                                                    Id);
 		}
-		update_app_panel(sel_year, sel_month, sel_day);
+		if (hilt_app == 0) hilt_app++;
 	}
 	erase_window_part(swin, 0, 0, nc_bar, nl_bar);
-	status_bar(which_pan, colr, nc_bar, nl_bar);
-	doupdate();
 }
 
 /* Updates the ToDo panel */
@@ -977,6 +1180,7 @@ void update_todo_panel(void)
 	int todo_lines = 1;
 	int max_items = nl_tod - 4;
 	int incolor = -1;
+	char mesg[MAX_LENGTH] = "";
 
 	/* Print todo item in the panel. */
 	erase_window_part(twin, 1, title_lines, nc_tod - 2, nl_tod - 2);
@@ -986,7 +1190,9 @@ void update_todo_panel(void)
 		incolor = num_todo - hilt_tod;
 		if (incolor == 0) saved_t_mesg = i->mesg; 
 		if (t_realpos >= 0 && t_realpos < max_items) {
-			display_item(twin, incolor, i->mesg,
+			sprintf(mesg, "%d. ", i->id);	
+			strncat(mesg, i->mesg, strlen(i->mesg));
+			display_item(twin, incolor, mesg,
 					len, y_offset, x_offset);
 			y_offset = y_offset + todo_lines;	
 		}
@@ -1023,7 +1229,7 @@ void update_app_panel(int year, int month, int day)
 	title_xpos = nc_app - (strlen(_(monthnames[sel_month - 1])) + 11);
 	if (sel_day < 10) title_xpos++;
 	date = date2sec(year, month, day, 0, 0);
-	write_app_pad(date, app_width, app_length, hilt_app, colr);
+	day_write_pad(date, app_width, app_length, hilt_app, colr);
 
 	/* Print current date in the top right window corner. */
 	erase_window_part(awin, 1, title_lines, nc_app - 2, nl_app - 2);
@@ -1049,7 +1255,7 @@ void update_app_panel(int year, int month, int day)
 	wnoutrefresh(awin);
 	pnoutrefresh(apad->ptrwin, apad->first_onscreen, 0, 
 		y_app + title_lines + 1, x_app + bordr, 
-		y_app + nl_app - 2*bordr, x_app + nc_app - 2*bordr);
+		y_app + nl_app - 2*bordr, x_app + nc_app - 3*bordr);
 }
 
 /*
@@ -1065,71 +1271,16 @@ void store_day(int year, int month, int day, bool day_changed)
 	/* Inits */
 	if (apad->length != 0)
 		delwin(apad->ptrwin);
-	if (adayptr != 0)
-		free_aday();
-	if (edayptr != 0)
-		free_eday();
 
-	/* Store the events and appointments. */
-	number_events_inday = eday_store(date);
-	number_apoints_inday = aday_store(date);
+	/* Store the events and appointments (recursive and normal items). */
+	apad->length = day_store_items(date, 
+		&number_events_inday, &number_apoints_inday);
 
 	/* Create the new pad with its new length. */
-	apad->length = number_events_inday + 1 + 3*number_apoints_inday;
 	if (day_changed) apad->first_onscreen = 0;
 	apad->ptrwin = newpad(apad->length, apad->width);
 }
 
-/* 
- * Write the appointments and events for the selected day in a pad.
- * An horizontal line is drawn between events and appointments, and the
- * item selected by user is highlighted.
- */
-void write_app_pad(long date, int width, int length, int incolor, int colr)
-{
-	struct event_s *e;
-	struct apoint_s *a;
-	int line = 0;
-	int item_number = 0;
-	int max_pos = 0;
-	const int x_pos = 0;
-
-	max_pos = length;
-	
-	/* First print the events for current day. */
-	for (e = edayptr; e != 0; e = e->next) {
-		item_number++;
-		if (item_number - incolor == 0){
-			event_hilt = true;
-			saved_e_mesg = e->mesg;
-		}
-		display_item(apad->ptrwin, item_number - incolor, e->mesg, 
-			width - 4, line, x_pos);
-		line++;
-	}	
-
-	/* Next draw a line between events and appointments. */
-	if (line > 0){
-		wmove(apad->ptrwin, line, x_pos);
-		whline(apad->ptrwin, 0, width);
-	}
-
-	/* Last print the appointments for current day. */
-	for (a = adayptr; a != 0; a = a->next) {
-		item_number++;
-		if (item_number - incolor == 0){ 
-			event_hilt = false;
-			apoint_sec2str(a, date, saved_a_start, saved_a_end);
-			saved_a_mesg = a->mesg;
-		}
-		display_item_date(apad->ptrwin, item_number - incolor, a,
-			date, line + 1, x_pos);	
-		display_item(apad->ptrwin, item_number - incolor, a->mesg,
-			width - 6, line + 2, x_pos + 2);
-		line = line + 3;
-	}
-}
-
 /* Load the user configuration */
 void load_conf(void)
 {
@@ -1147,6 +1298,7 @@ void load_conf(void)
 		wgetch(swin);
 	}
 	var = 0;
+	pthread_mutex_lock(&nbar->mutex);
 	for (;;) {
 		if (fgets(buf, 99, data_file) == NULL) {
 			break;
@@ -1183,6 +1335,19 @@ void load_conf(void)
 		} else if (var == 8) {
 			layout = atoi(e_conf);
 			var = 0;
+		} else if (var == 9) {
+			nbar->show = 
+				fill_config_var(e_conf);
+			var = 0;
+		} else if (var == 10) {
+			strncpy(nbar->datefmt, e_conf, strlen(e_conf) + 1);
+			var = 0;
+		} else if (var == 11) {
+			strncpy(nbar->timefmt, e_conf, strlen(e_conf) + 1);
+			var = 0;
+		} else if (var == 12) {
+			nbar->cntdwn = atoi(e_conf);
+			var = 0;
 		}
 		if (strncmp(e_conf, "auto_save=", 10) == 0)
 			var = 1;
@@ -1200,8 +1365,17 @@ void load_conf(void)
 			var = 7;
 		else if (strncmp(e_conf, "layout=", 7) == 0)
 			var = 8;
+		else if (strncmp(e_conf, "notify-bar_show=", 16) ==0)
+			var = 9;
+		else if (strncmp(e_conf, "notify-bar_date=", 16) ==0)
+			var = 10;
+		else if (strncmp(e_conf, "notify-bar_clock=", 17) ==0)
+			var = 11;
+		else if (strncmp(e_conf, "notify-bar_warning=", 19) ==0)
+			var = 12;
 	}
 	fclose(data_file);
+	pthread_mutex_unlock(&nbar->mutex);
 	erase_window_part(swin, 0, 0, nc_bar, nl_bar);
 }
 
