@@ -1,3 +1,5 @@
+/*	$calcurse: calendar.c,v 1.1.1.1 2006/07/31 21:00:02 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -29,6 +31,7 @@
 #include <time.h>
 
 #include "i18n.h"
+#include "day.h"
 #include "apoint.h"
 #include "event.h"
 #include "calendar.h"
@@ -47,11 +50,9 @@ update_cal_panel(WINDOW *cwin, int nl_cal,
                  bool monday_first)
 {
 	int c_day, c_day_1, day_1_sav, numdays, j;
-	struct apoint_s *i;
-	struct event_s *ie;
 	unsigned yr, mo;
 	int ofs_x, ofs_y;
-	int apoint_this_day;
+	int item_this_day = 0;
 	int title_lines = 3;
         int sunday_first = 0;
 
@@ -95,22 +96,9 @@ update_cal_panel(WINDOW *cwin, int nl_cal,
 	day_1_sav = (c_day_1 + 1) * 3 + c_day_1 - 7;
 
 	for (c_day = 1; c_day <= numdays; ++c_day, ++c_day_1, c_day_1 %= 7) {
-		apoint_this_day = 0;
 		//check if the day contains an event or an appointment
-		for (ie = eventlist; ie != 0; ie = ie->next)
-			if (event_inday
-			    (ie, date2sec(sel_year, sel_month, c_day, 0, 0))){
-				apoint_this_day = 1;
-				break;
-			}
-		if (apoint_this_day == 0){
-			for (i = apointlist; i != 0; i = i->next)
-				if (apoint_inday
-			    (i, date2sec(sel_year, sel_month, c_day, 0, 0))) {
-				apoint_this_day = 1;
-				break;
-			}
-		}
+		item_this_day = day_check_if_item(sel_year, sel_month, c_day);
+
 		/* Go to next line, the week is over. */
 		if (!c_day_1 && 1 != c_day) {	
 			++ofs_y;
@@ -141,7 +129,7 @@ update_cal_panel(WINDOW *cwin, int nl_cal,
 				 ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
 				 c_day);
 			custom_remove_attr(cwin, ATTR_MIDDLE);
-		} else if (apoint_this_day == 1) {
+		} else if (item_this_day) {
 			custom_apply_attr(cwin, ATTR_LOW);
 			mvwprintw(cwin, ofs_y + 1,
 				 ofs_x + day_1_sav + 4 * c_day + 1, "%2d",
