@@ -1,3 +1,5 @@
+/*	$calcurse: custom.c,v 1.3 2006/09/15 15:43:54 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -89,12 +91,14 @@ void config_bar()
 	mvwprintw(swin, 1, 2, "G");
 	mvwprintw(swin, 0, 2 + spc, "L");
 	mvwprintw(swin, 1, 2 + spc, "C");
+	mvwprintw(swin, 0, 2 + 2*spc, "N");
 	custom_remove_attr(swin, ATTR_HIGHEST);
 
 	mvwprintw(swin, 0, 2 + smlspc, _("Exit"));
 	mvwprintw(swin, 1, 2 + smlspc, _("General"));
 	mvwprintw(swin, 0, 2 + spc + smlspc, _("Layout"));
 	mvwprintw(swin, 1, 2 + spc + smlspc, _("Color"));
+	mvwprintw(swin, 0, 2 + 2*spc + smlspc, _("Notify"));
 	
 	wnoutrefresh(swin);
         wmove(swin, 0, 0);
@@ -106,9 +110,11 @@ int layout_config(int layout, int colr)
 {
 	int ch, old_layout;
 	char *layout_mesg = _("Pick the desired layout on next screen [press ENTER]");
-	char *choice_mesg = _("('A'= Appointment panel, 'c'= calendar panel, 't'= todo panel)");
-	char *layout_up_mesg   = _("   |Ac|      |At|      |cA|      |tA|");
-	char *layout_down_mesg = _("[1]|At|   [2]|Ac|   [3]|tA|   [4]|cA|");
+	char *choice_mesg = _("('A'= Appointment panel, 'C'= calendar panel, 'T'= todo panel)");
+	char *layout_up_mesg   = 
+	_("    AC       AT       CA       TA       TC       TA       CT       AT");
+	char *layout_down_mesg = 
+	_(" [1]AT    [2]AC    [3]TA    [4]CA    [5]TA    [6]TC    [7]AT    [8]CT");
 
 	old_layout = layout;
 	status_mesg(layout_mesg, choice_mesg);
@@ -117,18 +123,8 @@ int layout_config(int layout, int colr)
 	wnoutrefresh(swin);
 	doupdate();
 	while ((ch = wgetch(swin)) != 'q') {
-		switch (ch) {
-		case '1':
-			layout = 1;
-			return layout;
-		case '2':
-			layout = 2;
-			return layout;
-		case '3':
-			layout = 3;
-			return layout;
-		case '4':
-			layout = 4;
+		if ( ch <= '8' && ch >= '1' ) {
+			layout = ch - '0';
 			return layout;
 		}
 	}
