@@ -1,3 +1,5 @@
+/*	$calcurse: event.c,v 1.1.1.1 2006/07/31 21:00:03 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -33,7 +35,6 @@
 #include "utils.h"
 
 struct event_s *eventlist;
-struct event_s *edayptr;
 
 /* Create a new event */
 struct event_s *event_new(char *mesg, long day, int id)
@@ -56,27 +57,6 @@ struct event_s *event_new(char *mesg, long day, int id)
 	return o;
 }
 
-/* Add an event in the current day list */
-struct event_s *edayadd(char *mesg, long day, int id)
-{
-	struct event_s *o, **i;
-	o = (struct event_s *) malloc(sizeof(struct event_s));
-	o->mesg = (char *) malloc(strlen(mesg) + 1);
-	strcpy(o->mesg, mesg);
-	o->day = day;
-	o->id = id;
-	i = &edayptr;
-	for (;;) {
-		if (*i == 0 || (*i)->day > day) {
-			o->next = *i;
-			*i = o;
-			break;
-		}
-		i = &(*i)->next;
-	}
-	return o;
-}
-
 /* Check if the event belongs to the selected day */
 unsigned event_inday(struct event_s *i, long start)
 {
@@ -98,41 +78,6 @@ void event_write(struct event_s *o, FILE * f)
 		lt->tm_mon + 1, lt->tm_mday, 1900 + lt->tm_year, o->id, o->mesg);
 }
 
-/* 
- * Store the events for the selected day in structure pointed
- * by edayptr. This is done by copying the events 
- * from the general structure pointed by eventlist to the structure
- * dedicated to the selected day. 
- * Returns the number of events for the selected day.
- */
-int eday_store(long date)
-{
-	struct event_s *ptr, *j;
-	int e_nb = 0;
-
-	for (j = eventlist; j != 0; j = j->next) {
-		if (event_inday(j, date)) {
-			e_nb++;
-			ptr = edayadd(j->mesg, j->day, j->id);
-		}	
-	}
-
-	return e_nb;
-}
-
-/* Free the edayptr structure. */
-void free_eday(void)
-{
-	struct event_s *p, *q;
-
-	for (p = edayptr; p != 0; p = q) {
-		q = p->next;
-		free(p->mesg);
-		free(p);
-	}
-	edayptr = NULL;
-}
-
 /* Load the events from file */
 struct event_s *event_scan(FILE * f, struct tm start, int id)
 {
