@@ -1,3 +1,5 @@
+/*	$calcurse: event.h,v 1.1.1.1 2006/07/31 21:00:03 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -36,14 +38,10 @@ struct event_s {
 };
 
 extern struct event_s *eventlist;
-extern struct event_s *edayptr;
 
 struct event_s *event_new(char *, long, int);
-struct event_s *edayadd(char *mesg, long day, int id);
 unsigned event_inday(struct event_s *o, long start);
 void event_write(struct event_s *o, FILE * f);
-int eday_store(long date);
-void free_eday(void);
 struct event_s *event_scan(FILE * f, struct tm start, int id);
 void event_delete_bynum(long start, unsigned num);
 
