@@ -1,3 +1,5 @@
+/*	$calcurse: help.c,v 1.9 2006/09/17 18:24:39 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -33,6 +35,8 @@
 #include "custom.h"
 #include "vars.h"
 #include "utils.h"
+#include "apoint.h"
+#include "notify.h"
 
 /* 
  * Write the desired help text inside the help pad, and return the number
@@ -71,16 +75,14 @@ void help_screen(int which_pan, int colr)
 	WINDOW *help_pad = NULL;
 	char label[80];
 	int ch = '?';
-	int help_row = row - 2;	//size of the help window
+	int help_row, text_lines;
 	int help_col = col;
         int title_lines = 3;
 	int pad_offset = 4;
-	int text_lines = help_row - (pad_offset + 1);
 	int pad_width = help_col - 2*pad_offset + 1; 
 	int first_line = 0, nl = 0;
 	
 	help_page_t help_main;
-	help_page_t help_redraw;
 	help_page_t help_save;
 	help_page_t help_displacement;
 	help_page_t help_view;
@@ -88,7 +90,11 @@ void help_screen(int which_pan, int colr)
 	help_page_t help_goto;
 	help_page_t help_delete;
 	help_page_t help_add;
+	help_page_t help_priority;
+	help_page_t help_repeat;
 	help_page_t help_config;
+	help_page_t help_general;
+	help_page_t help_other;
 	help_page_t help_credits;
 
 	help_main.title = 
@@ -107,15 +113,6 @@ void help_screen(int which_pan, int colr)
     "                 corresponding action.\n\n"
     "       Credits:  Press '@' for credits.");
 
-	help_redraw.title = _("Redraw:\n");
-	help_redraw.text  =
-    _("Pressing 'R' redraws the Calcurse panels.\n\n"
-    "You might want to use this function when you resize your terminal\n"
-    "screen for example, and you want Calcurse to take into account the new\n"
-    "size of the terminal.\n\n"
-    "This function can also be useful when garbage appears in the display,\n"
-    "and you want to clean it.");
-
 	help_save.title = _("Save:\n");
     	help_save.text  =
     _("Pressing 'S' saves the Calcurse data.\n\n"
@@ -180,6 +177,9 @@ void help_screen(int which_pan, int colr)
     "\nDepending on which panel is selected when you press the delete key,\n"
     "the hilighted item of either the ToDo or Appointment list will be \n"
     "removed from this list.\n"
+    "\nIf the item to be deleted is recurrent, you will be asked if you\n"
+    "wish to suppress all of the item occurences or just the one you\n"
+    "selected.\n"
     "\nIf the general option 'confirm_delete' is set to 'YES', then you will\n"
     "be asked for confirmation before deleting the selected event.\n"
     "Do not forget to save the calendar data to retrieve the modifications\n"
@@ -189,8 +189,12 @@ void help_screen(int which_pan, int colr)
 	help_add.text  =
     _("Pressing 'A' allows you to add an item in either the ToDo or Appointment\n"
     "list, depending on which panel is selected when you press 'A'.\n"
-    "\nTo enter a new item in the TODO list, you only need to enter the\n"
-    "description of this new item.\n"
+    "\nTo enter a new item in the TODO list, you will need first to enter the\n"
+    "description of this new item. Then you will be asked to specify the todo\n"
+    "priority. This priority is represented by a number going from 9 for the\n"
+    "lowest priority, to 1 for the highest one. It is still possible to\n" 
+    "change the item priority afterwards, by using the '+/-' keys inside the\n"
+    "todo panel.\n"
     "\nIf the APPOINTMENT panel is selected while pressing 'A', you will be\n"
     "able to enter either a new appointment or a new all-day long event.\n"
     "To enter a new event, press [ENTER] instead of the item start time, and\n"
@@ -214,6 +218,43 @@ void help_screen(int which_pan, int colr)
     "       o do not forget to save the calendar data to retrieve the new\n"
     "         event next time you launch Calcurse.");
 
+	help_priority.title = _("Priority:\n");
+	help_priority.text  =
+    _("Pressing '+' or '-' allows you to change the priority of the currently\n"
+    "selected item in the ToDo list. Priorities are represented by the number\n"
+    "appearing in front of the todo description. This number goes from 9 for\n"
+    "the lowest priority to 1 for the highest priority. Todo having higher\n"
+    "priorities are placed first (at the top) inside the todo panel.\n\n"
+    "If you want to raise the priority of a todo item, you need to press '+'.\n"
+    "In doing so, the number in front of this item will decrease, meaning its\n"
+    "priority increases. The item position inside the todo panel may change,\n"
+    "depending on the priority of the items above it.\n\n"
+    "At the opposite, to lower a todo priority, press '-'. The todo position\n"
+    "may also change depending on the priority of the items below."); 
+
+	help_repeat.title = _("Repeat:\n");
+	help_repeat.text  =
+    _("Pressing 'R' allows you to repeat an event or an appointment. You must\n"
+    "first select the item to be repeated by moving inside the appointment\n"
+    "panel. Then pressing 'R' will lead you to a set of three questions, with\n"
+    "which you will be able to specify the repetition characteristics:\n\n"
+    "  o        type: you can choose between a daily, weekly, monthly or\n"
+    "                 yearly repetition by pressing 'D', 'W', 'M' or 'Y'\n"
+    "                 respectively.\n\n"
+    "  o   frequence: this indicates how often the item shall be repeated.\n"
+    "                 For example, if you want to remember an anniversary,\n"
+    "                 choose a 'yearly' repetition with a frequence of '1',\n"
+    "                 which means it must be repeated every year. Another\n"
+    "                 example: if you go to the restaurant every two days,\n"
+    "                 choose a 'daily' repetition with a frequence of '2'.\n\n" 
+    "  o ending date: this specifies when to stop repeating the selected\n"
+    "                 event or appointment. To indicate an endless \n"
+    "                 repetition, enter '0' and the item will be repeated\n"
+    "                 forever.\n"
+    "\nNote: the 'Repeat' and 'Delete' command can be mixed to create\n"
+    "        complicated configurations, as it is possible to delete only\n"
+    "        one occurence of a repeated item."); 
+
 	help_config.title = _("Config:\n");
 	help_config.text  =
     _("Pressing 'C' leads to the configuration submenu, from which you can\n"
@@ -223,9 +264,37 @@ void help_screen(int which_pan, int colr)
     "words where to place the three different panels on the screen.\n"
     "\nThe general options submenu brings a screen with the different options\n"
     "which modifies the way Calcurse interacts with the user.\n"
+    "\nThe notify submenu allows you to change the notify-bar settings.\n"
     "\nDo not forget to save the calendar data to retrieve your configuration\n"
     "next time you launch Calcurse.");
 
+	help_general.title = _("General keybindings:\n");
+	help_general.text  =
+    _("Some of the keybindings apply whatever panel is selected. They are\n"
+    "called general keybinding and involve the <CONTROL> key, which is\n"
+    "represented by the '^' sign in the status bar panel. For example,\n"
+    "'^A' means you need to press <CONTROL> and 'A' simultaneously to\n"
+    "activate the command. Here is the list of all the general keybindings,\n"
+    "together with their corresponding action:\n\n"
+    " '^R' : Redraw function -> redraws calcurse panels, this is useful if\n"
+    "                           you resize your terminal screen or when\n"
+    "                           garbage appears inside the display\n"
+    " '^A' : Add Appointment -> add an appointment or an event\n"
+    " '^T' : Add ToDo        -> add a todo\n"
+    " '^H' : -1 Day          -> move to previous day\n"
+    " '^L' : +1 Day          -> move to next day\n"
+    " '^K' : -1 Week         -> move to previous week\n"
+    " '^J' : +1 Week         -> move to next week");
+
+	help_other.title = _("OtherCmd:\n");
+	help_other.text  =
+    _("Pressing 'O' allows you to switch between status bar help pages.\n"
+    "Because the terminal screen is too narrow to display all of the\n"
+    "available commands, you need to press 'O' to see the next set of\n"
+    "commands together with their keybindings.\n"
+    "Once the last status bar page is reached, pressing 'O' another time\n"
+    "leads you back to the first page.");
+
 	help_credits.title = _("Calcurse - text-based organizer");
 	help_credits.text  =
     _("Copyright (c) 2004-2006 Frederic Culot\n"
@@ -246,22 +315,14 @@ void help_screen(int which_pan, int colr)
 	 * Create the help window and panel. The panel is used to make
 	 * the scrolling faster.
 	 */
+	help_row = (notify_bar()) ? row - 3 : row - 2; 
+	text_lines = help_row - (pad_offset + 1);
 	help_win = newwin(help_row, help_col, 0, 0);
 	help_pad = newpad(MAX_LENGTH, pad_width);
 	box(help_win, 0, 0);
 	sprintf(label, _("CalCurse %s | help"), VERSION);
 	win_show(help_win, label);
 
-	/* Display the main help screen. */
-/*	nl = write_help_pad(help_pad, help_main.title, help_main.text, pad_width);
-	status_bar(which_pan, colr, col, 2);
-        wmove(swin, 0, 0);
-	wnoutrefresh(help_win);
-	pnoutrefresh(help_pad, first_line, 0, pad_offset, pad_offset, 
-			help_row - 2, help_col - pad_offset);
-        doupdate();
-*/
-
 	/* Display the help screen related to user input. */
 	while ( ch != 'q' ) {
                 erase_window_part(help_win, 1, title_lines, 
@@ -269,11 +330,11 @@ void help_screen(int which_pan, int colr)
 
 		switch (ch) {
 		
-		case CTRL('N') :
+		case CTRL('n') :
 			if (nl > first_line + text_lines) first_line++;
 			break;
 
-		case CTRL('P') :
+		case CTRL('p') :
 			if (first_line > 0) first_line--;
 			break;
 
@@ -282,10 +343,16 @@ void help_screen(int which_pan, int colr)
 			nl = write_help_pad(help_pad, help_main.title,
 					help_main.text, pad_width);
 			break;
-		case 'r':
+		case CTRL('r'):
+		case CTRL('a'):
+		case CTRL('t'):
+		case CTRL('h'):
+		case CTRL('j'):
+		case CTRL('k'):
+		case CTRL('l'):
 			first_line = 0;
-			nl = write_help_pad(help_pad, help_redraw.title,
-					help_redraw.text, pad_width);
+			nl = write_help_pad(help_pad, help_general.title,
+					help_general.text, pad_width);
 			break;
 		case 's':
 			first_line = 0;
@@ -328,6 +395,16 @@ void help_screen(int which_pan, int colr)
 			nl = write_help_pad(help_pad, help_config.title,
 					help_config.text, pad_width);
 			break;
+		case 'o':
+			first_line = 0;
+			nl = write_help_pad(help_pad, help_other.title,
+					help_other.text, pad_width);
+			break;
+		case 'r':
+			first_line = 0;
+			nl = write_help_pad(help_pad, help_repeat.title,
+					help_repeat.text, pad_width);
+			break;
 
 		case 'v':
 			first_line = 0;
@@ -335,6 +412,13 @@ void help_screen(int which_pan, int colr)
 					help_view.text, pad_width);
 			break;
 
+		case '+':
+		case '-':
+			first_line = 0;
+			nl = write_help_pad(help_pad, help_priority.title,
+				help_priority.text, pad_width);
+			break;
+
 		case 9:
 			first_line = 0;
 			nl = write_help_pad(help_pad, help_tab.title,
@@ -355,6 +439,8 @@ void help_screen(int which_pan, int colr)
 			int highend = (int) (ratio * first_line);
 			int sbar_top = highend + title_lines + 1;
 
+			if ((sbar_top + sbar_length) > help_row - 1)
+				sbar_length = help_row - 1 -sbar_top;
 			draw_scrollbar(help_win, sbar_top, help_col - 2,
 					sbar_length, title_lines + 1, 
 					help_row - 1, true);
@@ -367,6 +453,7 @@ void help_screen(int which_pan, int colr)
 				help_row - 2, help_col - pad_offset);
                 doupdate();
                 ch = wgetch(swin);
+		ch = tolower(ch);
 	}
 	delwin(help_pad);
 	delwin(help_win);
