@@ -1,3 +1,5 @@
+/*	$calcurse: io.c,v 1.7 2006/09/15 15:43:03 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -23,6 +25,7 @@
  */
 
 #include <ncurses.h>
+#include <pthread.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -39,6 +42,7 @@
 #include "todo.h"
 #include "event.h"
 #include "apoint.h"
+#include "recur.h"
 #include "io.h"
 #include "vars.h"
 
@@ -121,8 +125,8 @@ save_cal(bool auto_save, bool confirm_quit,
 	 int colr, int layout)
 {
 	FILE *data_file;
-        struct event_s *k;
-	struct apoint_s *j;
+	struct event_s *k;
+	apoint_llist_node_t *j;
 	struct todo_s *i;
 	char *access_pb = _("Problems accessing data file ...");
 	char *config_txt =
@@ -187,6 +191,30 @@ save_cal(bool auto_save, bool confirm_quit,
 			"\n# This is the layout of the calendar (1 to 4) :\n");
 		fprintf(data_file, "layout=\n");
 		fprintf(data_file, "%d\n", layout);
+
+		// notify-bar user settings
+		pthread_mutex_lock(&nbar->mutex);
+		fprintf(data_file,
+			"\n# If this option is set to yes, notify-bar will be displayed :\n");
+		fprintf(data_file, "notify-bar_show=\n");
+		fprintf(data_file, "%s\n", (nbar->show) ? "yes" : "no");
+		
+		fprintf(data_file,
+			"\n# Format of the date to be displayed inside notify-bar :\n");
+		fprintf(data_file, "notify-bar_date=\n");
+		fprintf(data_file, "%s\n", nbar->datefmt);
+
+		fprintf(data_file,
+			"\n# Format of the time to be displayed inside notify-bar :\n");
+		fprintf(data_file, "notify-bar_clock=\n");
+		fprintf(data_file, "%s\n", nbar->timefmt);
+
+		fprintf(data_file,
+			"\n# Warn user if he has an appointment within next 'notify-bar_warning' seconds :\n");
+		fprintf(data_file, "notify-bar_warning=\n");
+		fprintf(data_file, "%d\n", nbar->cntdwn);
+		pthread_mutex_unlock(&nbar->mutex);
+
 		fclose(data_file);
 	}
 
@@ -197,21 +225,27 @@ save_cal(bool auto_save, bool confirm_quit,
 	        status_mesg(access_pb, ""); 
 	else {
 		for (i = todolist; i != 0; i = i->next)
-			fprintf(data_file, "%s\n", i->mesg);
+			fprintf(data_file, "[%d] %s\n", i->id, i->mesg);
 		fclose(data_file);
 	}
 
 	/* 
          * Save the apts data file, which contains the 
          * appointments first, and then the events. 
+	 * Recursive items are written first.
          */
 	if (show_bar) progress_bar(save, 3);
 	data_file = fopen(path_apts, "w");
 	if (data_file == (FILE *) 0)
 	        status_mesg(access_pb, "");
 	else {
-		for (j = apointlist; j != 0; j = j->next)
+		recur_save_data(data_file);
+
+		pthread_mutex_lock(&(alist_p->mutex));
+		for (j = alist_p->root; j != 0; j = j->next)
 			apoint_write(j, data_file);
+		pthread_mutex_unlock(&(alist_p->mutex));
+
 		for (k = eventlist; k != 0; k = k->next)
 			event_write(k, data_file);
 		fclose(data_file);
@@ -227,25 +261,29 @@ save_cal(bool auto_save, bool confirm_quit,
 
 /* 
  * Check what type of data is written in the appointment file, 
- * and then load either a new appointment or a new event.
+ * and then load either: a new appointment, a new event, or a new
+ * recursive item (which can also be either an event or an appointment).
  */ 
 void load_app()
 {
 	FILE *data_file;
-	int c, is_appointment, is_event;
-        struct tm start, end, *lt;
+	int c, is_appointment, is_event, is_recursive;
+        struct tm start, end, until, *lt;
+	struct days_s *exc = NULL;
         time_t t;
         int id = 0;
+	int freq;
+	char type;
+	char *error = 
+		_("FATAL ERROR in load_app: wrong format in the appointment or event\n");
 
         t = time(NULL);
         lt = localtime(&t);
-        start = *lt;
-        end = *lt;
-
+        start = end = until = *lt;
+	
 	data_file = fopen(path_apts, "r");
 	for (;;) {
-                is_appointment = 0;
-                is_event = 0;
+                is_appointment = is_event = is_recursive = 0;
 		c = getc(data_file);
 		if (c == EOF)
 			break;
@@ -279,19 +317,85 @@ void load_app()
 
                 /* Read the remaining informations. */
                 if (is_appointment) {
-                        fscanf(data_file, "@ %u : %u -> %u / %u / %u @ %u : %u |",
+                        fscanf(data_file, "@ %u : %u -> %u / %u / %u @ %u : %u ",
                                &start.tm_hour, &start.tm_min,
                                &end.tm_mon, &end.tm_mday, &end.tm_year,
                                &end.tm_hour, &end.tm_min); 
-                        apoint_scan(data_file, start, end);
                 } else if (is_event) {
                         fscanf(data_file, "[%d] ", &id);
-                        event_scan(data_file, start, id);
                 } else { /* NOT REACHED */
-                    		fputs(_("FATAL ERROR in load_app: wrong format "
-					"in the appointment or event\n"), stderr);
-		exit(EXIT_FAILURE);
+                  	fputs(error, stderr);
+			exit(EXIT_FAILURE);
                 }
+		
+		/* Check if we have a recursive item. */
+		c = getc(data_file);
+
+		if (c == '{') {
+			ungetc(c, data_file);
+			is_recursive = 1;
+			fscanf(data_file, "{ %d%c ", &freq, &type);
+			/* Check if we have an endless recurrent item. */
+			c = getc(data_file);
+			if (c == '}') {
+				ungetc(c, data_file);
+				fscanf(data_file, "} ");
+				until.tm_year = 0;	
+				if (is_appointment) 
+					c = getc(data_file); // useless '|'
+			} else if (c == '-') {
+				ungetc(c, data_file);
+				fscanf(data_file, " -> %u / %u / %u ",
+					&until.tm_mon, &until.tm_mday, 
+					&until.tm_year);
+				c = getc(data_file);
+				if (c == '!') {
+					ungetc(c, data_file);
+					exc = recur_exc_scan(data_file);
+					fscanf(data_file, "} ");
+				} else {
+					ungetc(c, data_file);
+					fscanf(data_file, "} ");
+				}
+				if (is_appointment) 
+					fscanf(data_file, " | "); // useless '|'
+			} else if (c == '!') { // endless item with exceptions
+				ungetc(c, data_file);
+				exc = recur_exc_scan(data_file);
+				if (is_appointment) 
+					fscanf(data_file, " | "); // useless '|'
+				until.tm_year = 0;
+			} else { /* NOT REACHED */
+				fputs(error, stderr);
+				exit(EXIT_FAILURE);
+			}
+		} else {
+			if (is_event) // if appointment we have a useless '|' 
+				ungetc(c, data_file);
+		}
+
+		/*
+		 * Last: read the item description and load it into its
+		 * corresponding linked list, depending on the item type.
+		 */
+		if (is_appointment) {
+			if (is_recursive) {
+				recur_apoint_scan(data_file, start, end,
+					type, freq, until, exc);
+			} else {
+				apoint_scan(data_file, start, end);
+			}
+		} else if (is_event) {
+			if (is_recursive) {
+				recur_event_scan(data_file, start, id,
+					type, freq, until, exc);
+			} else {
+				event_scan(data_file, start, id);
+			}
+		} else { /* NOT REACHED */
+			fputs(error, stderr);
+			exit(EXIT_FAILURE);	
+		}
 	}
 	fclose(data_file);
 }
@@ -303,7 +407,9 @@ load_todo(int colr)
 	FILE *data_file;
 	char *mesg_line1 = _("Failed to open todo file");
 	char *mesg_line2 = _("Press [ENTER] to continue");
+	char *nl;
 	int nb_tod = 0;
+	int c, id;
 	char buf[100], e_todo[100];
 
 	data_file = fopen(path_todo, "r");
@@ -312,11 +418,22 @@ load_todo(int colr)
 		wgetch(swin);
 	}
 	for (;;) {
-		if (fgets(buf, 99, data_file) == NULL) {
+		c = getc(data_file);
+		if (c == EOF) {
 			break;
+		} else if (c == '[') { /* new style with id */
+			fscanf(data_file, "%d] ", &id);
+		} else {
+			id = 9;
+			ungetc(c, data_file);
+		}
+		fgets(buf, MAX_LENGTH, data_file);
+		nl = strchr(buf, '\n');
+		if (nl) {
+			*nl = '\0';
 		}
 		extract_data(e_todo, buf, strlen(buf));
-		todo_add(e_todo);
+		todo_add(e_todo, id);
 		++nb_tod;
 	}
 	fclose(data_file);
@@ -330,7 +447,6 @@ int check_data_files()
 	int no_data_file;
 
 	no_data_file = 0;
-	erase_window_part(swin, 0, 0, col, 2);
 	/* Create the calcurse repertory if not present. */
 	mkdir(path_dir, 0700);
 
