@@ -1,3 +1,5 @@
+/*	$calcurse: todo.h,v 1.3 2006/08/31 18:49:05 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -28,12 +30,16 @@
 struct todo_s {
 	struct todo_s *next;
 	char *mesg;
+	int id;
 };
 
 extern struct todo_s *todolist;
 
-struct todo_s *todo_insert(char *mesg);
-struct todo_s *todo_add(char *mesg);
+int todo_new_item(int total, int colr); 
+struct todo_s *todo_add(char *mesg, int id);
 void todo_delete_bynum(unsigned num);
+struct todo_s *todo_get_item(int item_number);
+int todo_get_position(struct todo_s *i);
+int todo_chg_priority(int action, int item_num);
 
 #endif /* CALCURSE_TODO_H */
