@@ -1,3 +1,5 @@
+/*	$calcurse: utils.h,v 1.5 2006/09/16 15:25:36 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -25,6 +27,23 @@
 #ifndef CALCURSE_UTILS_H
 #define CALCURSE_UTILS_H
 
+#define MAX(x,y) 	((x)>(y)?(x):(y))
+#define MIN(x,y) 	((x)<(y)?(x):(y))
+
+#define NB_CAL_CMDS	15 /* number of commands while in cal view */
+#define NB_APP_CMDS	18 /* same thing while in appointment view */ 
+#define NB_TOD_CMDS	18 /* same thing while in todo view */
+#define TOTAL_CMDS	NB_CAL_CMDS + NB_APP_CMDS + NB_TOD_CMDS
+#define NB_PANELS	3  /* 3 panels: CALENDAR, APPOINTMENT, TODO */
+#define CMDS_PER_LINE	6  /* max number of commands per line */
+#define KEY_LENGTH	4  /* length of each keybinding + one space */
+#define LABEL_LENGTH	8  /* length of command description */
+
+typedef struct { /* structure defining a keybinding */
+	char *key; 
+	char *label;
+} binding_t;
+
 void status_mesg(char *mesg_line1, char *mesg_line2);
 void erase_window_part(WINDOW *win, int first_col, int first_row, 
                        int last_col, int last_row);
@@ -49,5 +68,10 @@ void item_in_popup(char *saved_a_start, char *saved_a_end, char *msg,
 void win_show(WINDOW * win, char *label);
 void display_item(WINDOW *win, int incolor, char *msg, 
 		int len, int y, int x);
+void reset_status_page(void);
+void other_status_page(int panel);
+long today(void);
+long now(void);
+char *mycpy(const char *src);
 
 #endif /* CALCURSE_UTILS_H */
