@@ -1,3 +1,5 @@
+/*	$calcurse: vars.c,v 1.2 2006/09/15 15:38:14 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -81,3 +83,6 @@ WINDOW *awin = NULL, *cwin = NULL, *twin = NULL, *swin = NULL;
 
 /* Variable to handle pads. */
 struct pad_s *apad;
+
+/* Variable to store notify-bar settings. */
+struct nbar_s *nbar;
