@@ -1,3 +1,5 @@
+/*	$calcurse: vars.h,v 1.4 2006/09/15 15:37:45 culot Exp $	*/
+
 /*
  * Calcurse - text-based organizer
  * Copyright (c) 2004-2006 Frederic Culot
@@ -26,6 +28,7 @@
 #define CALCURSE_VARS_H
 
 #include <ncurses.h> 
+#include <pthread.h>
 #include <stdbool.h>
 
 #define DIR_NAME	".calcurse"
@@ -36,6 +39,7 @@
 #define MAX_LENGTH	512
 
 #define CTRL(x)         ((x) & 0x1f)
+#define ESCAPE		27
 
 #define ATTR_FALSE	0
 #define ATTR_TRUE	1
@@ -45,6 +49,8 @@
 #define ATTR_HIGH	5
 #define ATTR_HIGHEST	6
 
+#define DAYINSEC  	86400 
+
 struct pad_s {
 	int width;
 	int length;
@@ -52,6 +58,14 @@ struct pad_s {
 	WINDOW *ptrwin;		/* pointer to the pad window */
 };
 
+struct nbar_s {		
+	int show;		/* display or hide the notify-bar */
+	int cntdwn;		/* warn when time left before next app < cntdwn */  
+	char *datefmt;		/* format for displaying date */
+	char *timefmt;		/* format for displaying time */
+	pthread_mutex_t mutex;
+};
+
 extern int col, row;
 extern bool colorize;
 extern int days[12];
@@ -63,5 +77,6 @@ extern char path_apts[MAX_LENGTH];
 extern char path_conf[MAX_LENGTH];
 extern WINDOW *awin, *cwin, *twin, *swin;
 extern struct pad_s *apad;
+extern struct nbar_s *nbar;
 
 #endif /* CALCURSE_VARS_H */
